"""
This module implements a set of :class:`revscoring.Language`
-- collections of features that are language specific.

languages
+++++++++
.. automodule:: revscoring.languages.dutch

.. automodule:: revscoring.languages.english

.. automodule:: revscoring.languages.estonian

.. automodule:: revscoring.languages.french

.. automodule:: revscoring.languages.german

.. automodule:: revscoring.languages.hebrew

.. automodule:: revscoring.languages.indonesian

.. automodule:: revscoring.languages.italian

.. automodule:: revscoring.languages.persian

.. automodule:: revscoring.languages.portuguese

.. automodule:: revscoring.languages.spanish

.. automodule:: revscoring.languages.turkish

.. automodule:: revscoring.languages.ukrainian

.. automodule:: revscoring.languages.vietnamese

Base classes
++++++++++++
.. automodule:: revscoring.languages.language

.. automodule:: revscoring.languages.space_delimited

"""
from .language import Language

__all__ = [Language]
