"""
This module implements a set of utilities for extracting features and
train/testing :class:`revscoring.scorer_models.MLScorerModel` from the
command-line.  When the revscoring python package is installed, a `revscoring`
utility should be available from the commandline.  Run `revscoring -h` for more
information:

extract_features
++++++++++++++++
.. automodule:: revscoring.utilities.extract_features

score
+++++
.. automodule:: revscoring.utilities.score

train_test
++++++++++
.. automodule:: revscoring.utilities.train_test
"""
